#include "findMotif.h"

char progress[4] = {'-','\\', '|', '/'};
int status=0;

bool FindMotif::Test_Rev = true;
int FindMotif::Num_Seed = 100;
double FindMotif::Pseudo_Prob=0.001;

/*****************************************************************************/
/*                                                                           */
/* Constructor and Destructor                                                */
/*                                                                           */
/*****************************************************************************/
FindMotif::FindMotif( const int _motif_len,
                      const double _max_binding_ratio,
                      vector<DNA*> &true_dna_set,
                      vector<DNA*> &false_dna_set ) :
  motif_len (_motif_len),
  max_binding_ratio (_max_binding_ratio),
  num_true_seq (0),
  num_false_seq (0),
  toolsPtr (NULL)
{
  for ( int i=0;i<Num_Motif;i++)
    motifArr[i]=NULL;

  for ( vector<DNA*>::iterator i=true_dna_set.begin();
        i < true_dna_set.end();
        i++) {
    true_dna_vec.push_back(*i);
    num_true_seq++;
    //printf("Added: %s\n", (*i)->getSeq() );
  }
  for ( vector<DNA*>::iterator i=false_dna_set.begin();
        i < false_dna_set.end();
        i++) {
    false_dna_vec.push_back(*i);
    num_false_seq++;
  }

  int total_len = 0;
  for ( int i = num_true_seq-1; i>=0; i--)
    total_len += strlen(true_dna_vec[i]->getSeq());
  for ( int i = num_false_seq-1; i>=0; i--)
    total_len += strlen(false_dna_vec[i]->getSeq());
  avg_seq_len = total_len / (num_true_seq + num_false_seq);
}

FindMotif::~FindMotif() {
  for(int i=0; i<(int)true_dna_vec.size();i++)
    delete true_dna_vec.at(i);
  for(int i=0; i<(int)false_dna_vec.size();i++)
    delete false_dna_vec.at(i);
  for(int i=0;i<Num_Motif;i++)
    if(motifArr[i]!=NULL)
      delete motifArr[i];
  delete toolsPtr;
}

/*****************************************************************************/
/*                                                                           */
/* Pub 1 Getting data                                                        */
/*                                                                           */
/*****************************************************************************/

//get the set of DNA regions
//format:
// >Name and information
// ACGTACGTACGTACGT
void FindMotif::getDNASeq(vector<DNA*> &dnaVector, const string filePath, const int maxIn) {  
  ifstream inDNAFile( filePath.c_str(), ios::in);

  if(!inDNAFile) {
    cerr << "The file "+filePath+" cannot be opened" <<endl;
    exit(1);
  }
  
  int count=0;
  char dnaSeq[Max_DNA_Len];
  char dnaName[Max_Seed_Len];
  
  while (inDNAFile.getline(dnaName,Max_Seed_Len) && count < maxIn) {
    inDNAFile.getline(dnaSeq,Max_DNA_Len);
    count++;
    
    int nameLength;
    int dnaLength;
    if((dnaLength=strlen(dnaSeq)) > 0) {
      
      nameLength=strlen(dnaName);
      if(isspace(dnaName[nameLength-1]))
        nameLength--;
        
      char *dnaNamePtr=new char[nameLength+1];
      if(dnaNamePtr==NULL){
        cout<<"In FindMotif::getDNASeq"<<endl;
        cout<<"Not enough memory for dnaNamePtr"<<endl;
        exit(1);
      }
      
      for(int i=0;i<nameLength;i++)
        dnaNamePtr[i]=dnaName[i];
      dnaNamePtr[nameLength]=0;
    
      if(isspace(dnaSeq[dnaLength-1]))
        dnaLength--;
        
      char *dnaSeqPtr=new char[dnaLength+1];
      if(dnaNamePtr==NULL){
        cout<<"In FindMotif::getDNASeq"<<endl;
        cout<<"No enough memory for dnaSeqPtr"<<endl;
        exit(1);
      }
      for(int i=0;i<dnaLength;i++)
        dnaSeqPtr[i]=toupper(dnaSeq[i]);
      dnaSeqPtr[dnaLength]=0;
    
      dnaVector.push_back(new DNA(dnaNamePtr,dnaSeqPtr));
    }
  }
  
  inDNAFile.close();
}

void FindMotif::getInputDNASeq() {
  getDNASeq(true_dna_vec, trueSetFilePath, maxNumOfSeq);
  getDNASeq(false_dna_vec, falseSetFilePath, maxNumOfSeq);
  
  avg_seq_len=0;
  for(int i=0; i<(int)true_dna_vec.size();i++)
    avg_seq_len+=strlen(true_dna_vec.at(i)->getSeq());
  for(int i=0; i<(int)false_dna_vec.size();i++)
    avg_seq_len+=strlen(false_dna_vec.at(i)->getSeq());
  
  avg_seq_len/=(true_dna_vec.size()+false_dna_vec.size());
}

/*****************************************************************************/
/*                                                                           */
/* Pub 1 Algorithm                                                           */
/*                                                                           */
/*****************************************************************************/


/*****************************************/
/* Pub 1.1 Finding motif                 */
/*****************************************/

void FindMotif::discover_seed(const long motifLength, //const long maxHamDist,
                              const long maxBindSite, const long outputSize,
			      vector<Motif*> & motifVec,
			      const bool testRev,
			      const bool isFast) {
  
  if(isFast){
    //if the motif length is very small
    if(motifLength<=max_neigh_pat_len){
      discover_seedShort(motifLength, hamming_dist, maxBindSite, outputSize,
                         motifVec, testRev);
    } else if(motifLength<=max_mem_pat_len) {
      discover_seedMed(motifLength, hamming_dist, maxBindSite, outputSize,
	  	     motifVec, testRev);
    } else
      discover_seedLong(motifLength, hamming_dist, maxBindSite, outputSize,
	  	      motifVec, testRev);
  } else { 
    //find seed by brute force
    int seedLength=(int)min((double)FindMotif::maxSeedLength, (double)motifLength);
    int seedSize=(int)pow(4.0,seedLength);
    
    toolsPtr=new Tools(num_true_seq, (int)min((double)num_false_seq,(double)noOfFUsedInSeed)
                       , maxBindSite, motifLength, avg_seq_len);
    if(toolsPtr==NULL){
      cout<<"In FindMotif::findSeed"<<endl;
      cout<<"No enough memory for toolsPtr"<<endl;
      exit(1);
    }
    
    //find the score of each candiate matrix
    BindingSitePosition** scoreArr=new BindingSitePosition*[seedSize];
    if(scoreArr==NULL){
      cout<<"In FindMotif::findSeed"<<endl;
      cout<<"No enough memory for scoreArr"<<endl;
      exit(1);
    }
     
    //for each seed, calculate the p-value 
    double* tempProbMat;
    for(int i=0;i<seedSize;i++) {
      
      if(i%1024==0)
        cout<<"tuning the "<<i<<" seed"<<endl;
        
      tempProbMat=new double[4*motifLength];
      if(tempProbMat==NULL){
        cout<<"In FindMotif::findSeed"<<endl;
        cout<<"No enough memory for tempProbMat"<<endl;
        exit(1);
      }  
      Tools::numToMat(i, tempProbMat, motifLength, seedLength);

      Motif* tempMotif=EMLoop(tempProbMat, noOfFUsedInSeed, motifLength, seedTunedTurn);

      scoreArr[i]=new BindingSitePosition(tempMotif->getScore(),i);
      if(scoreArr[i]==NULL){
        cout<<"In FindMotif::findSeed"<<endl;
        cout<<"No enough memory for scoreArr["<<i<<"]"<<endl;
        exit(1);
      }
      delete tempMotif;
    }
    Tools::sortBSP_D(scoreArr, seedSize);

    char motifStr[seedLength+1];
  
    for(int seedNo=0;seedNo<seedSize;seedNo++) {  
      if((int)motifVec.size() == outputSize)
        break;
    
      int value=(scoreArr[seedSize-seedNo-1])->getSeqNo();	//get the pattern number;
      for(int i=0;i<seedLength;i++) {
        switch(value%4) {
          case 0:
            motifStr[i]='A';
            break;
          case 1:
            motifStr[i]='C';
            break;
          case 2:
            motifStr[i]='G';
            break;
          case 3:
            motifStr[i]='T';
            break;
          default:
            cout<<"In FindMotif::findSeed"<<endl;
            cout<<"Error in switch, value = "<<value<<", value%4 = "<<value%4<<endl;
            exit(1);
        }
        value/=4;
      }
      motifStr[seedLength]=0;
    
      //test if the string occurs (with error)
      bool exist=false;
      for(int i=0;i<(int)motifVec.size();i++) {
        int count=0;
        for(int j=0;j<seedLength;j++)
          if(motifStr[j]!=((motifVec.at(i))->get_pattern())[j])
            count++;
        if(count<=seedSelectHD){
          exist=true;
          break;
        }
      }
    
      //add the string
      if(!exist) {
        double* tempProbMat=new double[4*motifLength];
        Tools::numToMat((scoreArr[seedSize-seedNo-1])->getSeqNo(), tempProbMat, motifLength, seedLength);
        Motif* tempMotif=EMLoop(tempProbMat, noOfFUsedInSeed, motifLength, seedTunedTurn);   	
        motifVec.push_back(tempMotif);
      }
      
    }
    
    //deallocate array
    delete[] scoreArr;
    delete toolsPtr;  
  }
}

void FindMotif::discover_seedShort(const long motifLength, const long maxHamDist,
                                   const long maxBindSite, const long outputSize,
				   vector<Motif*> & motifVec, 
				   const bool testRev) {
  Neighbor neighborTbl(motifLength, maxHamDist);
  neighborTbl.createNeighborList();

  long size=(long)pow(4.0,(int)motifLength);
  long* scoreTbl=new long[size];
  if(scoreTbl==NULL){
    cout<<"In FindMotif::discover_seedShort"<<endl;
    cout<<"No enough memory for scoreTbl"<<endl;
    exit(1);
  }
  for(long i=0;i<size;i++)
    scoreTbl[i]=0;
  for(long seqNo=0;seqNo<(long)true_dna_vec.size();seqNo++) {
    const char* dnaSeq=true_dna_vec.at(seqNo)->getSeq();
    for(long i=0;i<=(long)strlen(dnaSeq)-motifLength;i++) {
      unsigned long patternNum=Tools::patternToLong(&dnaSeq[i],motifLength);
      for(long d=0;d<=maxHamDist;d++){
        const unsigned long* maskArr=neighborTbl.getMaskArr(d);
        long size=neighborTbl.getMaskNo(d);
        for(long j=0;j<size;j++)
          scoreTbl[patternNum ^ maskArr[j]]++;
      }
    }
  }

  if(testRev) {
    for(long seqNo=0;seqNo<(long)true_dna_vec.size();seqNo++) {
      const char* dnaSeq=true_dna_vec.at(seqNo)->getSeqR();
      for(long i=0;i<=(long)strlen(dnaSeq)-motifLength;i++) {
        unsigned long patternNum=Tools::patternToLong(&dnaSeq[i],motifLength);
        for(long d=0;d<=maxHamDist;d++){
          const unsigned long* maskArr=neighborTbl.getMaskArr(d);
          long size=neighborTbl.getMaskNo(d);
          for(long j=0;j<size;j++)
            scoreTbl[patternNum ^ maskArr[j]]++;
        }
      }
    }
  }
  //linear search to find the highest score patterns
  unsigned long* candiateArr=new unsigned long[outputSize];
  if(candiateArr==NULL){
    cout<<"In FindMotif::discover_seedShort"<<endl;
    cout<<"No enough memory for candiateArr"<<endl;
    exit(1);
  }
  long* candScoreArr=new long[outputSize];
  if(candScoreArr==NULL){
    cout<<"In FindMotif::discover_seedShort"<<endl;
    cout<<"No enough memory for candScoreArr"<<endl;
    exit(1);
  }
  for(long i=0;i<outputSize;i++) {
    unsigned long candiate=0;
    long maxScore=scoreTbl[0];

    for(unsigned long j=1;j<(unsigned long)size;j++) {
      if(scoreTbl[j]>maxScore) {
        maxScore=scoreTbl[j];
        candiate=j;
      }
    }
    candiateArr[i]=candiate;
    candScoreArr[i]=maxScore;
    scoreTbl[candiate]=0;
  }

  for(long i=0;i<outputSize;i++) {
    char* motifPat=new char[motifLength+1];
    Tools::longToPattern(motifPat, motifLength,candiateArr[i]);
    //motifVec[i]=new Motif(motifPat,candScoreArr[i]);
    motifVec.push_back( new Motif(motifPat,candScoreArr[i]) );
  }

  //disallocate memory
  delete[] scoreTbl;
  delete[] candiateArr;
}

void FindMotif::discover_seedMed(const long motifLength, const long maxHamDist,
                                 const long maxBindSite, const long outputSize,
				 vector<Motif*> & motifVec,
				 const bool testRev) {
  
  long firstHalfLen=motifLength/2;
  long secondHalfLen=motifLength-firstHalfLen;
  Neighbor* neighborFirst=new Neighbor(firstHalfLen, maxHamDist);
  neighborFirst->createNeighborList();
                                                                                
  Neighbor* neighborSecond;
  if(firstHalfLen==secondHalfLen)
    neighborSecond=neighborFirst;
  else {
    neighborSecond=new Neighbor(secondHalfLen, maxHamDist);
    neighborSecond->createNeighborList();
  }
                                                                                
  long size=(long)pow(4.0,(int)motifLength);
                                                                                
  long* scoreTbl=new long[size];
  if(scoreTbl==NULL){
    cout<<"In FindMotif::discover_seedMed"<<endl;
    cout<<"No enough memory for scoreTbl"<<endl;
    exit(1);
  }
                                                                                
  for(long i=0;i<size;i++)
    scoreTbl[i]=0;
                                                                                
  for(long seqNo=0;seqNo<(long)true_dna_vec.size();seqNo++) {
    const char* dnaSeq=true_dna_vec.at(seqNo)->getSeq();
                                                                                
    for(long i=0;i<=(long)strlen(dnaSeq)-motifLength;i++) {
      unsigned long patternNum=Tools::patternToLong(&dnaSeq[i],motifLength);
                                                                                
      for(long d1=0;d1<=maxHamDist;d1++){
        const unsigned long* maskArr1=neighborFirst->getMaskArr(d1);
        long size1=neighborFirst->getMaskNo(d1);
                                                                                
        for(long d2=0;d2<=maxHamDist-d1;d2++){
          const unsigned long* maskArr2=neighborSecond->getMaskArr(d2);
          long size2=neighborSecond->getMaskNo(d2);
                                                                                
          for(long j=0;j<size1;j++) {
            unsigned long mask = maskArr1[j] << (secondHalfLen*2);
                                                                                
            for(int k=0;k<size2;k++) {
              scoreTbl[(patternNum ^ mask)^maskArr2[k]]++;
            }
          }
        }
      }
    }
  }
  if(testRev) {
    for(long seqNo=0;seqNo<(long)true_dna_vec.size();seqNo++) {
      const char* dnaSeq=true_dna_vec.at(seqNo)->getSeqR();
                                                                                
      for(long i=0;i<=(long)strlen(dnaSeq)-motifLength;i++) {
        unsigned long patternNum=Tools::patternToLong(&dnaSeq[i],motifLength);
                                                                                
        for(long d1=0;d1<=maxHamDist;d1++){
          const unsigned long* maskArr1=neighborFirst->getMaskArr(d1);
          long size1=neighborFirst->getMaskNo(d1);
                                                                                
          for(long d2=0;d2<=maxHamDist-d1;d2++){
            const unsigned long* maskArr2=neighborSecond->getMaskArr(d2);
            long size2=neighborSecond->getMaskNo(d2);
                                                                                
            for(long j=0;j<size1;j++) {
              for(int k=0;k<size2;k++) {
                unsigned long mask = maskArr1[j] << (secondHalfLen*2);
                scoreTbl[(patternNum ^ mask)^maskArr2[k]]++;
              }
            }
          }
        }
     }
    }
  }
 //linear search to find the highest score patterns
  unsigned long* candiateArr=new unsigned long[outputSize];
  if(candiateArr==NULL){
    cout<<"In FindMotif::discover_seedMed"<<endl;
    cout<<"No enough memory for candiateArr"<<endl;
    exit(1);
  }
  long* candScoreArr=new long[outputSize];
  if(candScoreArr==NULL){
    cout<<"In FindMotif::discover_seedMed"<<endl;
    cout<<"No enough memory for candScoreArr"<<endl;
    exit(1);
  }
                                                                                
  for(long i=0;i<outputSize;i++) {
    unsigned long candiate=0;
    long maxScore=scoreTbl[0];
                                                                                
    for(unsigned long j=1;j<(unsigned long)size;j++) {
      if(scoreTbl[j]>maxScore) {
        maxScore=scoreTbl[j];
        candiate=j;
      }
    }
                                                                                
    candiateArr[i]=candiate;
    candScoreArr[i]=maxScore;
    scoreTbl[candiate]=0;
  }

  for(long i=0;i<outputSize;i++) {
    char* motifPat=new char[motifLength+1];
    Tools::longToPattern(motifPat, motifLength,candiateArr[i]);
    //motifVec[i]=new Motif(motifPat, candScoreArr[i]);
    motifVec.push_back(new Motif(motifPat, candScoreArr[i]));
  }
  
  //disallocate memory
  delete[] scoreTbl;
  delete[] candiateArr;
  
  if(neighborFirst!=NULL)
    delete neighborFirst;
  if(firstHalfLen!=secondHalfLen)
    delete neighborSecond;
}

void FindMotif::discover_seedLong(const long motifLength, const long maxHamDist,
                                  const long maxBindSite, const long outputSize,
				  vector<Motif*> & motifVec,
				  const bool testRev) {

  long firstHalfLen=max_mem_pat_len/2;
  long secondHalfLen=max_mem_pat_len-firstHalfLen;
  Neighbor* neighborFirst=new Neighbor(firstHalfLen, maxHamDist);
  neighborFirst->createNeighborList();

  Neighbor* neighborSecond;
  if(firstHalfLen==secondHalfLen)
    neighborSecond=neighborFirst;
  else {
    neighborSecond=new Neighbor(secondHalfLen, maxHamDist);
    neighborSecond->createNeighborList();
  }

  long fillInSize=(long)pow(4.0,(int)max_mem_pat_len);

  long* scoreTbl=new long[fillInSize];
  if(scoreTbl==NULL){
    cout<<"In FindMotif::discover_seedLong"<<endl;
    cout<<"No enough memory for scoreTbl"<<endl;
    exit(1);
  }

  Motif** candMotifArr=new Motif*[outputSize];
  if(candMotifArr==NULL){
    cout<<"In FindMotif::discover_seedLong"<<endl;
    cout<<"No enough memory for candMotifArr"<<endl;
    exit(1);
  }

  Motif** tempCandMotifArr=new Motif*[outputSize];
  if(tempCandMotifArr==NULL){
    cout<<"In FindMotif::discover_seedLong"<<endl;
    cout<<"No enough memory for tempCandMotifArr"<<endl;
    exit(1);
  }
  int suffixSize=motifLength-max_mem_pat_len;
  char* suffixPat=new char[suffixSize+1];
  for(unsigned long suffixNo=0;suffixNo<(unsigned long)pow(4.0,suffixSize);suffixNo++) {
    cout<<"suffixNo="<<suffixNo<<endl;
    Tools::longToPattern(suffixPat,suffixSize,suffixNo);
    for(long i=0;i<fillInSize;i++)
      scoreTbl[i]=0;
    for(long seqNo=0;seqNo<(long)true_dna_vec.size();seqNo++) {
      const char* dnaSeq=true_dna_vec.at(seqNo)->getSeq();
      for(long i=0;i<=(long)strlen(dnaSeq)-motifLength;i++) {
        unsigned long patternNum=Tools::patternToLong(&dnaSeq[i],max_mem_pat_len);
        long suffixHamDist=Tools::calHamDist(suffixPat, &dnaSeq[i+max_mem_pat_len], suffixSize);
	
	for(long d1=0;d1<=maxHamDist-suffixHamDist;d1++){
          const unsigned long* maskArr1=neighborFirst->getMaskArr(d1);
          long size1=neighborFirst->getMaskNo(d1);
	  
          for(long d2=0;d2<=maxHamDist-suffixHamDist-d1;d2++){
            const unsigned long* maskArr2=neighborSecond->getMaskArr(d2);
            long size2=neighborSecond->getMaskNo(d2);
	    
            for(long j=0;j<size1;j++) {
              unsigned long mask = maskArr1[j] << (secondHalfLen*2);
	      
              for(int k=0;k<size2;k++) {
                scoreTbl[(patternNum ^ mask)^maskArr2[k]]++;
              }
            }
          }
        }
      }
    }
    if(testRev){
      for(long seqNo=0;seqNo<(long)true_dna_vec.size();seqNo++) {
        const char* dnaSeq=true_dna_vec.at(seqNo)->getSeqR();
	
        for(long i=0;i<=(long)strlen(dnaSeq)-motifLength;i++) {
          unsigned long patternNum=Tools::patternToLong(&dnaSeq[i],max_mem_pat_len);
          long suffixHamDist=Tools::calHamDist(suffixPat, &dnaSeq[max_mem_pat_len]
                                               , suffixSize);
	  
          for(long d1=0;d1<=maxHamDist-suffixHamDist;d1++){
            const unsigned long* maskArr1=neighborFirst->getMaskArr(d1);
            long size1=neighborFirst->getMaskNo(d1);
	    
            for(long d2=0;d2<=maxHamDist-suffixHamDist-d1;d2++){
              const unsigned long* maskArr2=neighborSecond->getMaskArr(d2);
              long size2=neighborSecond->getMaskNo(d2);
	      
              for(long j=0;j<size1;j++) {
                unsigned long mask = maskArr1[j] << (secondHalfLen*2);
                for(int k=0;k<size2;k++) {
                  scoreTbl[(patternNum ^ mask)^maskArr2[k]]++;
                }
              }
            }
          }
        }
      }
    }
    
    //linear search to find the highest score patterns in current suffix
    if(suffixNo==0) {
      for(long i=0;i<outputSize;i++) {
        unsigned long candiate=0;
        long maxScore=scoreTbl[0];
	
        for(unsigned long j=1;j<(unsigned long)fillInSize;j++) {
          if(scoreTbl[j]>maxScore) {
            maxScore=scoreTbl[j];
            candiate=j;
          }
        }
        scoreTbl[candiate]=0;
	
        char* motifPat=new char[motifLength+1];
        Tools::longToPattern(motifPat, max_mem_pat_len,candiate);
        candMotifArr[i]=new Motif(strcat(motifPat,suffixPat),maxScore);
      }
    } else {
      long pos1=0;
      long pos2=0;
      
      for(long i=0;i<outputSize;i++) {
        unsigned long candiate=0;
        long maxScore=scoreTbl[0];
	
        for(unsigned long j=1;j<(unsigned long)fillInSize;j++) {
          if(scoreTbl[j]>maxScore) {
            maxScore=scoreTbl[j];
            candiate=j;
          }
        }
        scoreTbl[candiate]=0;
        if(candMotifArr[pos1]->getScore() >= maxScore) {
          tempCandMotifArr[i]=candMotifArr[pos1];
          pos1++;
        } else {
          char* motifPat=new char[motifLength+1];
          Tools::longToPattern(motifPat, max_mem_pat_len,candiate);
          strcat(motifPat,suffixPat);
          candMotifArr[outputSize-pos2-1]->set_pattern(motifPat);
          candMotifArr[outputSize-pos2-1]->set_score(maxScore);
          tempCandMotifArr[i]=candMotifArr[outputSize-pos2-1];
          pos2++;
        }
      }
      
      Motif** tempArr=candMotifArr;
      candMotifArr=tempCandMotifArr;
      tempCandMotifArr=tempArr;
    }
  } //end of for each suffixNo
  
  for(int i=0;i<outputSize;i++)
    motifVec.push_back(candMotifArr[i]);
  
  //disallocate memory
  delete[] scoreTbl;
  delete[] suffixPat;
  delete candMotifArr;
  delete tempCandMotifArr;
  
  if(neighborFirst!=NULL)
    delete neighborFirst;
  if(firstHalfLen!=secondHalfLen)
    delete neighborSecond;
}

void str_to_prob_mat(const char *str, const int motif_len, double * prob_mat) {
  for (int i=0; i<motif_len; i++) {
    if ( str[i] == 'A' ) {
      prob_mat[i] = 0.5;
    } else {
      prob_mat[i] = 0.5 / 3.0;
    }
  }
  int offset = motif_len;
  for (int i=0; i<motif_len; i++) {
    if ( str[i] == 'C' ) {
      prob_mat[offset+i] = 0.5;
    } else {
      prob_mat[offset+i] = 0.5 / 3.0;
    }
  }
  offset += motif_len;
  for (int i=0; i<motif_len; i++) {
    if ( str[i] == 'G' ) {
      prob_mat[offset+i] = 0.5;
    } else {
      prob_mat[offset+i] = 0.5 / 3.0;
    }
    
  }
  offset += motif_len;
  for (int i=0; i<motif_len; i++) {
    if ( str[i] == 'T' ) {
      prob_mat[offset+i] = 0.5;
    } else {
      prob_mat[offset+i] = 0.5 / 3.0;
    }    
  }
}

void str_to_prob_mat2(const char *str, const int motif_len, double * prob_mat) {
  int preSpace=(motif_len-strlen(str))/2;
  
  for(int i=0;i<4*motif_len;i++)
    prob_mat[i]=0.25;
  
  for (int i=0; i<(int)strlen(str); i++) {
    if ( str[i] == 'A' ) {
      prob_mat[i+preSpace] = 0.5;
    } else {
      prob_mat[i+preSpace] = 0.5 / 3.0;
    }
  }
  int offset = motif_len+preSpace;
  for (int i=0; i<motif_len; i++) {
    if ( str[i] == 'C' ) {
      prob_mat[offset+i] = 0.5;
    } else {
      prob_mat[offset+i] = 0.5 / 3.0;
    }
  }
  offset += motif_len;
  for (int i=0; i<(int)strlen(str); i++) {
    if ( str[i] == 'G' ) {
      prob_mat[offset+i] = 0.5;
    } else {
      prob_mat[offset+i] = 0.5 / 3.0;
    }
    
  }
  offset += motif_len;
  for (int i=0; i<(int)strlen(str); i++) {
    if ( str[i] == 'T' ) {
      prob_mat[offset+i] = 0.5;
    } else {
      prob_mat[offset+i] = 0.5 / 3.0;
    }    
  }
}

void FindMotif::discoverMotif(const int motifLength, const int noOfTurns,
			      vector<double*> &seedVec,const bool isFast) {
  for(int i=0;i<Num_Motif;i++) {
    if(motifArr[i]!=NULL) {
      delete motifArr[i];
      motifArr[i]=NULL;
    }
  }  
  // find the seed motifs
  //cout<<"begin finding seed"<<endl;
  //vector<double*> seedVec;

  seedVec.clear();

  vector <Motif *> seeds;

  discover_seed( motifLength,
		 (int)max_binding_ratio, FindMotif::Num_Seed,
		 seeds, Test_Rev, isFast);
  for ( vector<Motif *>::iterator i=seeds.begin();
        i < seeds.end();
        i++) {
    Motif *motif = (*i);

    //cerr << "Seed: " <<  motif->get_pattern() << endl;
    double * prob_mat = new double[4*motif_len];
    if(isFast)
      str_to_prob_mat(motif->get_pattern().c_str(), motif_len, prob_mat);
    else{
      for(int j=0;j<4*motifLength;j++)
        prob_mat[j]=motif->getProbMat()[j];
    }
      
    seedVec.push_back(prob_mat);

#if 0
    for (int i=0; i<motif_len; i++)
      printf("%0.3f", prob_mat[i] );
    printf("\n");
    for (int i=0; i<motif_len; i++)
      printf("%0.3f", prob_mat[i+motif_len] );
    printf("\n");
    for (int i=0; i<motif_len; i++)
      printf("%0.3f", prob_mat[i+motif_len*2] );
    printf("\n");
    for (int i=0; i<motif_len; i++)
      printf("%0.3f", prob_mat[i+motif_len*3] );
    printf("\n");
    cout << endl << endl;
#endif
  }

  //printf("seedVec[] => size %d\n", seedVec.size());
  //findSeed(FindMotif::Num_Seed,motifLength,seedVec);
  
  //cerr << "Construct toolsPtr " << time(NULL) << endl;
  toolsPtr = new Tools(true_dna_vec.size(), false_dna_vec.size(),
		       max_binding_ratio, motifLength, avg_seq_len);
  //cerr << "constricted toolsPtr" << time(NULL) << endl;

  if(toolsPtr==NULL){
    cout<<"In FindMotif::discoverMotif"<<endl;
    cout<<"No enough memory for toolsPtr"<<endl;
    exit(1);
  }
  //cout<<"finished construct toolsPtr"<<endl;
  
  cout<<"\tBegin tuning the "<< seedVec.size() << " seeds\n\t"<<endl;
  
  //fprintf ( stderr, "\n\n\ttuning on the seeds... \n");

  Motif* tempMotif;
  for(int i=0; i<(int)seedVec.size();i++) {
    bool insert=false;
    //printf("Tuning seed %d\n", i);

    tempMotif=EMLoop(seedVec.at(i),-1, motifLength, noOfTurns);
    
#ifndef NDEBUG
    fprintf(stderr, "\r\tTuning seed %d",i);
    status = status % 4;
#endif
    
    //Test if the tuned motif should be inserted to the Arr;
    if(motifArr[Num_Motif-1]==NULL)
      insert=true;
    else {
      if(tempMotif->getScore() < motifArr[Num_Motif-1]->getScore()) {
        insert=true;
      }
    }
    
    if(insert) {  
      int insertPos=Num_Motif-1;
      
      while(insertPos>0) {
      	if(motifArr[insertPos-1]==NULL)
      	  insertPos--;
      	else{
      	  if(tempMotif->getScore() < motifArr[insertPos-1]->getScore())
            insertPos--;
          else
            if(tempMotif->getScore() == motifArr[insertPos-1]->getScore()
               && tempMotif->getAlpha() > motifArr[insertPos-1]->getAlpha())
              insertPos--;
            else
              break;
        }
      }
      
      if( motifArr[Num_Motif-1]!=NULL)
        delete motifArr[Num_Motif-1];
      
      for(int j=Num_Motif-1;j>insertPos;j--)
        motifArr[j]=motifArr[j-1];
      motifArr[insertPos]=tempMotif;
    }
    else
      delete tempMotif;
  }

  fprintf(stderr, "\n\n");
}

void FindMotif::output(vector<double*> &seedVec, ostream &os) {
//void FindMotif::output(vector<Motif *> &seedVec, ostream &os) {
  
  int outputSize=seedVec.size();
  if((int)seedVec.size()>Num_Motif)
    outputSize=Num_Motif;
  char motif_str[motif_len+1];
  
  int trueSetSize = true_dna_vec.size();  
  int falseSetSize = false_dna_vec.size();
  
  for(int i=0;i<outputSize;i++) {   
    printf ("\n motif #%d :\n", i);
    
    for(int j=0;j<4;j++) {
      for(int k=0; k<motif_len; k++) {
        printf (" %2.2f ",
		(motifArr[i]->getProbMat() )[k + j*motif_len]);
      }
      printf ("\n");
    }
    
    Tools::matToStr(motifArr[i]->getProbMat(), motif_str, motif_len);
    cout<<"pattern : "<<motif_str<<endl;
    cout<<"log(p-value) : "<<motifArr[i]->getScore()<<endl;
    cout<<"alpha : "<<motifArr[i]->getAlpha()<<endl;
    cout<<"distribution of the binding sites in true set :"<<endl;
    
    int siteInT=0;
    for(int j=0;j<trueSetSize;j++)
      siteInT+=(motifArr[i]->getBSDistributeT())[j];
    cout<<siteInT<<":  ";
     
    for(int j=0;j<trueSetSize;j++)
      printf("%d ", (motifArr[i]->getBSDistributeT())[j] );
    printf("\n");

    cout<<"distribution of the binding sites in false set is"<<endl;
    int siteInF=0;
    for(int j=0;j<falseSetSize;j++)
      siteInF+=(motifArr[i]->getBSDistributeF())[j];
    printf("%d:  ", siteInF );
     
    for(int j=0;j<falseSetSize;j++)
      printf("%d ", (motifArr[i]->getBSDistributeF())[j] );
    printf("\n");

  }
}

/***********************************************/
/* Pub 1.3 EM tuning for a probability Matrix */
/***********************************************/ 
Motif* FindMotif::EMLoop(double* probMat, const int numOfFused
                         , const int motifLength, const int noOfTurns){
  //initate p value;
  double logPValue_min=2;	//min p-value for finding the suitable alpha in an iteration
  double alpha=0.0;			//best alpha in an iteration
  double bestPValue=2.0;		//min p-value for the whole tuning
  double bestAlpha=0.0;		//best alpha for the whole tuning
  
  //initate array
  double* bestMat=new double[4*motifLength];	//best matrix for the whole tuning
  if(bestMat==NULL){
    cout<<"In FindMotif::EMLoop"<<endl;
    cout<<"No enough memory for bestMat"<<endl;
    exit(1);
  }
  double* tunedMat=new double[4*motifLength];	//temp matrix for refining
  if(tunedMat==NULL){
    cout<<"In FindMotif::EMLoop"<<endl;
    cout<<"No enough memory for tunedMat"<<endl;
    exit(1);
  }
  
  int noOfBindingSite=0;
  int trueSetSize=true_dna_vec.size();
  int falseSetSize;
  if(numOfFused==-1)
    falseSetSize=false_dna_vec.size();
  else
    falseSetSize=(int)min((double)numOfFused,(double)false_dna_vec.size());
  
  //temp distribution  
  int* seqWithSiteArrT=new int[trueSetSize];
  if(seqWithSiteArrT==NULL){
    cout<<"In FindMotif::EMLoop"<<endl;
    cout<<"No enough memory for seqWithSiteArrT"<<endl;
    exit(1);
  }
  //temp distribution
  int* seqWithSiteArrF=new int[falseSetSize];
  if(seqWithSiteArrF==NULL){
    cout<<"In FindMotif::EMLoop"<<endl;
    cout<<"No enough memory for seqWithSiteArrF"<<endl;
    exit(1);
  }
  
  if(Test_Rev) {
    for(int i=0;i<trueSetSize;i++)
      if((int)noOfBindingSite < (int)(2*(strlen(true_dna_vec.at(i)->getSeq())-motifLength+1)) )
        noOfBindingSite=2*(strlen(true_dna_vec.at(i)->getSeq())-motifLength+1);
    for(int i=0;i<falseSetSize;i++)
      if((int)noOfBindingSite < (int)(2*(strlen(false_dna_vec.at(i)->getSeq())-motifLength+1) ) )
        noOfBindingSite=2*(strlen(false_dna_vec.at(i)->getSeq())-motifLength+1);
  }
  else {
    for(int i=0;i<trueSetSize;i++)
      if(noOfBindingSite < (int)(strlen(true_dna_vec.at(i)->getSeq())-motifLength+1 ) )
        noOfBindingSite=strlen(true_dna_vec.at(i)->getSeq())-motifLength+1;
    for(int i=0;i<falseSetSize;i++)
      if(noOfBindingSite < (int)strlen(false_dna_vec.at(i)->getSeq())-motifLength+1 )
        noOfBindingSite=strlen(false_dna_vec.at(i)->getSeq())-motifLength+1;
  }
  
  //score of all substring in a sequence for 
  BindingSitePosition** allSubstrScore
    = new BindingSitePosition*[noOfBindingSite];
  if(allSubstrScore==NULL){
    cout<<"In FindMotif::EMLoop"<<endl;
    cout<<"No enough memory for allSubstrScore"<<endl;
    exit(1);
  }

  BindingSitePosition** trueSetScoreTop=new BindingSitePosition*[trueSetSize*(int)max_binding_ratio];
  if(trueSetScoreTop==NULL){
    cout<<"In FindMotif::EMLoop"<<endl;
    cout<<"No enough memory for trueSetScoreTop"<<endl;
    exit(1);
  }
  BindingSitePosition** falseSetScoreTop=new BindingSitePosition*[falseSetSize*(int)max_binding_ratio];
  if(falseSetScoreTop==NULL){
    cout<<"In FindMotif::EMLoop"<<endl;
    cout<<"No enough memory for falseSetScoreTop"<<endl;
    exit(1);
  }
  
  BindingSitePosition** tempTop=new BindingSitePosition*[(int)max_binding_ratio];
  
  int* bsDistributeT=new int[trueSetSize];
  if(bsDistributeT==NULL){
    cout<<"In FindMotif::EMLoop"<<endl;
    cout<<"No enough memory for bsDistributeT"<<endl;
    exit(1);
  }
  int* bsDistributeF=new int[falseSetSize];
  if(bsDistributeF==NULL){
    cout<<"In FindMotif::EMLoop"<<endl;
    cout<<"No enough memory for bsDistributeF"<<endl;
    exit(1);
  }
  
  int* bestBSDistributeT=new int[trueSetSize];
  if(bestBSDistributeT==NULL){
    cout<<"In FindMotif::EMLoop"<<endl;
    cout<<"No enough memory for bestBSDistributeT"<<endl;
    exit(1);
  }
  int* bestBSDistributeF=new int[falseSetSize];
  if(bestBSDistributeF==NULL){
    cout<<"In FindMotif::EMLoop"<<endl;
    cout<<"No enough memory for bestBSDistributeF"<<endl;
    exit(1);
  }
  
  //EM step
  
  //calculating scores
  for(int step=0 ; step<noOfTurns ; step++) {    
    //get the score of the probability matrix
    
    //calculate the score of each true sequence
    for(int i=0;i<trueSetSize;i++) {
      int noOfBS=strlen(true_dna_vec.at(i)->getSeq())-motifLength+1;
      true_dna_vec.at(i)->calculateScore(probMat, motifLength, Test_Rev);
    
      if(Test_Rev) {
        for(int j=0;j<noOfBS;j++){
	  // --BS--
          BindingSitePosition* bsp = 
	    new BindingSitePosition( (true_dna_vec.at(i)->getScoreArr())[j],j);
	  // --BS--
          BindingSitePosition* bspR =
	    new BindingSitePosition( (true_dna_vec.at(i)->getScoreArrR())[j],noOfBS-j-1);
          allSubstrScore[2*j]=bsp;
          allSubstrScore[2*j+1]=bspR;
        }
        noOfBS*=2;
      }
      else {
      	for(int j=0;j<noOfBS;j++){
	  // --BS--
          BindingSitePosition* bsp=new BindingSitePosition( (true_dna_vec.at(i)->getScoreArr())[j],j);
          allSubstrScore[j]=bsp;
        }
      }
 
      Tools::sortBSP_D(allSubstrScore,noOfBS);
      
      int noOfInsert=1;
      // --BS--
      trueSetScoreTop[i*(int)max_binding_ratio + 0] =
	new BindingSitePosition(allSubstrScore[0]->getScore(),i);
      tempTop[0]=allSubstrScore[0];
      
      for(int j=1;j<noOfBS;j++) {
        bool close=false;
        for(int k=0;k<noOfInsert;k++) {
          if(abs(allSubstrScore[j]->getSeqNo() - tempTop[k]->getSeqNo() ) < motifLength) {
            close=true;
            break;
          }
        }
        
        if(!close){         
	  // --BS--
          trueSetScoreTop[i*(int)max_binding_ratio + noOfInsert] =
	    new BindingSitePosition(allSubstrScore[j]->getScore(),i);
          tempTop[noOfInsert]=allSubstrScore[j];
          noOfInsert++;
          if(noOfInsert>=(int)max_binding_ratio)
            break;
        }
      }
      for(int j=1;j<noOfBS;j++)
        delete allSubstrScore[j];
    }
    
    //calculate the score of each false sequence
    for(int i=0;i<falseSetSize;i++) {
      int noOfBS=strlen(false_dna_vec.at(i)->getSeq())-motifLength+1;
      false_dna_vec.at(i)->calculateScore(probMat, motifLength, Test_Rev);
    
      if(Test_Rev) {
        for(int j=0;j<noOfBS;j++){
	  // --BS--
          BindingSitePosition* bsp =
	    new BindingSitePosition( (false_dna_vec.at(i)->getScoreArr())[j],j);
	  // --BS--
          BindingSitePosition* bspR = 
	    new BindingSitePosition( (false_dna_vec.at(i)->getScoreArrR())[j],noOfBS-j-1);
          allSubstrScore[2*j]=bsp;
          allSubstrScore[2*j+1]=bspR;
        }
        noOfBS*=2;
      }
      else {
      	for(int j=0;j<noOfBS;j++){
	  // --BS--
          BindingSitePosition* bsp =
	    new BindingSitePosition( (false_dna_vec.at(i)->getScoreArr())[j],j);
          allSubstrScore[j]=bsp;
        }  
      }
 
      Tools::sortBSP_D(allSubstrScore,noOfBS);
      
      int noOfInsert=1;
      // --BS--
      falseSetScoreTop[i*(int)max_binding_ratio + 0] =
	new BindingSitePosition(allSubstrScore[0]->getScore(),i);
      tempTop[0]=allSubstrScore[0];
      
      for(int j=1;j<noOfBS;j++) {
        bool close=false;
        for(int k=0;k<noOfInsert;k++) {
          if(abs(allSubstrScore[j]->getSeqNo() - tempTop[k]->getSeqNo() ) < motifLength) {
            close=true;
            break;
          }
        }
        
        if(!close){
	  // --BS--
          falseSetScoreTop[i*(int)max_binding_ratio + noOfInsert]=
	    new BindingSitePosition(allSubstrScore[j]->getScore(),i);
          tempTop[noOfInsert]=allSubstrScore[j];
          noOfInsert++;
          if(noOfInsert>=(int)max_binding_ratio)
            break;
        }
      }

      for(int j=1;j<noOfBS;j++)
        delete allSubstrScore[j];
    }

    //sorting the scores
    Tools::sortBSP_D(trueSetScoreTop,trueSetSize*(int)max_binding_ratio);
    Tools::sortBSP_D(falseSetScoreTop,falseSetSize*(int)max_binding_ratio);

    //calculate alpha
    double tempAlpha;
    logPValue_min=2;
    double tempLogPValue=0;
    int noOfSeqWithBST=0;
    int noOfSeqWithBSF=0;
    
    int noOfFalseBS=0;
    
    for(int i=0;i<trueSetSize;i++)
      seqWithSiteArrT[i]=0;
    for(int i=0;i<falseSetSize;i++)
      seqWithSiteArrF[i]=0;
      
    //loop the possible alpha
    for(int i=0;i<trueSetSize*(int)max_binding_ratio;i++) {
      tempAlpha=trueSetScoreTop[i]->getScore();
      seqWithSiteArrT[trueSetScoreTop[i]->getSeqNo()]++;
      
      if(seqWithSiteArrT[trueSetScoreTop[i]->getSeqNo()]==1) {
        noOfSeqWithBST++;
      }
        
      while(noOfFalseBS < falseSetSize*(int)max_binding_ratio) {
      	if(falseSetScoreTop[noOfFalseBS]->getScore()>=tempAlpha) {
          seqWithSiteArrF[falseSetScoreTop[noOfFalseBS]->getSeqNo()]++;
          if(seqWithSiteArrF[falseSetScoreTop[noOfFalseBS]->getSeqNo()]==1) {
            noOfSeqWithBSF++;
          }
          noOfFalseBS++;
        }
        else
          break;
      }
      tempLogPValue=toolsPtr->score(i+1,noOfFalseBS);
            
      if(tempLogPValue < logPValue_min) {
        logPValue_min=tempLogPValue;
        alpha=tempAlpha;
        for(int j=0;j<trueSetSize;j++)
          bsDistributeT[j]=seqWithSiteArrT[j];
        for(int j=0;j<falseSetSize;j++)
          bsDistributeF[j]=seqWithSiteArrF[j];
      }
    }  

    //fprintf ( stderr, " |\b\b" );

    //cout<<">>>step = "<<step<<"\tlog p-value ="<<logPValue_min<<endl;
       
    //renew the matrix
    if(logPValue_min<bestPValue) {
      bestPValue=logPValue_min;
      bestAlpha=alpha;
      for(int i=0;i<4*motifLength;i++)
        bestMat[i]=probMat[i];
      for(int j=0;j<trueSetSize;j++)
        bestBSDistributeT[j]=bsDistributeT[j];
      for(int j=0;j<falseSetSize;j++)
        bestBSDistributeF[j]=bsDistributeF[j];
    }
    
    //refine the probability matrix
    if(step!=noOfTurns-1) {
      for(int i=0;i<4*motifLength;i++) {
        probMat[i]=0;
      }
       
      double totalP;
      double tempP;

      //for each sequence
      int totalSite=0; 
      for(int i=0;i<trueSetSize;i++) {      
        
        for(int j=0;j<4*motifLength;j++)
          tunedMat[j]=0.0;
        totalP=0.0;
        
        for(int j=0;j<(int)strlen(true_dna_vec.at(i)->getSeq())-motifLength+1;j++){
          //original sequence
          tempP=1.0/(1+exp(alpha - (true_dna_vec.at(i)->getScoreArr())[j] ));  
          totalP+=tempP;
          
          for(int k=0;k<motifLength;k++) {
            switch(true_dna_vec.at(i)->getSeq()[j+k]) {
              case 'A':
                tunedMat[k]+=tempP;
                break;
              case 'C':
                tunedMat[k+motifLength]+=tempP;
                break;
              case 'G':
                tunedMat[k+2*motifLength]+=tempP;
                break;
              case 'T':
                tunedMat[k+3*motifLength]+=tempP;
                break;
              default:
                cout<<"In FindMotif::EM"<<endl;
                cout<<"The "<<j+k<<"-th character in "<<true_dna_vec.at(i)->getName()
                    <<" is \'"<<(true_dna_vec.at(i)->getSeq())[i]
                    <<"\' which is not a valid character"<<endl;
                exit(1);
                break;
            }
          }
          
          //reversed sequence
          if(Test_Rev) {
            tempP=1.0/(1+exp(alpha - (true_dna_vec.at(i)->getScoreArrR())[j] ));   
            totalP+=tempP;
            for(int k=0;k<motifLength;k++) {
              switch(true_dna_vec.at(i)->getSeqR()[j+k]) {
                case 'A':
                  tunedMat[k]+=tempP;
                  break;
                case 'C':
                  tunedMat[k+motifLength]+=tempP;
                  break;
                case 'G':
                  tunedMat[k+2*motifLength]+=tempP;
                  break;
                case 'T':
                  tunedMat[k+3*motifLength]+=tempP;
                  break;
                default:
                  cout<<"In FindMotif::EM"<<endl;
                  cout<<"The "<<j+k<<"-th character in the reversed DNA of "<<true_dna_vec.at(i)->getName()
                      <<" is \'"<<(true_dna_vec.at(i)->getSeqR())[i]
                      <<"\' which is not a valid character"<<endl;
                  exit(1);
                  break;
              }
            }
          }
        }//end of for loop a sequence
        
        int noOfSite=(int)max(1.0, (double)bestBSDistributeT[i]);
        totalSite+=noOfSite;
        for(int j=0;j<4*motifLength;j++)
          probMat[j]+=tunedMat[j]/totalP*noOfSite;
      }
      for(int i=0;i<4*motifLength;i++) {
        probMat[i]=probMat[i]/totalSite;
        probMat[i]=(probMat[i]+FindMotif::Pseudo_Prob) / (1.0+4*FindMotif::Pseudo_Prob);
      }
    } //end of if
    
    //delete array
    for(int i=0;i<trueSetSize*(int)max_binding_ratio;i++)
      delete trueSetScoreTop[i];
    for(int i=0;i<falseSetSize*(int)max_binding_ratio;i++)
      delete falseSetScoreTop[i];
  }//end of EM step
  
  
  for(int i=0;i<4*motifLength;i++)
    probMat[i]=bestMat[i];
  Motif* motif=new Motif(probMat, bestPValue, bestBSDistributeT, bestBSDistributeF, bestAlpha);
  if(motif==NULL){
    cout<<"In FindMotif::EMLoop"<<endl;
    cout<<"No enough memory for motif"<<endl;
    exit(1);
  }
  
  //delete array
  delete [] bestMat;
  delete [] tunedMat;
  
  delete [] allSubstrScore;
  delete [] trueSetScoreTop;
  delete [] falseSetScoreTop;
  delete [] tempTop;
  
  delete [] seqWithSiteArrT;
  delete [] seqWithSiteArrF;
  delete [] bsDistributeT;
  delete [] bsDistributeF;
  
  return motif;
}           
             
/*****************************************************************************/
/*                                                                           */
/* Main function                                                             */
/*                                                                           */
/*****************************************************************************/
